# frozen_string_literal: true

require 'spec_helper'

require_relative '../../../lib/balboa/cli/parser'

describe Balboa::CLI::Parser do
  it 'parses a password option into a hash of configurations' do
    argv = ['-p', 'test']
    config = described_class.parse(argv)

    expect(config).to eq('password' => 'test')
  end

  it 'parses a password full option into a hash of configurations' do
    argv = ['--password', 'test']
    config = described_class.parse(argv)

    expect(config).to eq('password' => 'test')
  end

  it 'parses a project option into a hash of configurations' do
    argv = ['-w', 'test']
    config = described_class.parse(argv)

    expect(config).to eq('project' => 'test')
  end

  it 'parses a project full option into a hash of configurations' do
    argv = ['--project', 'test']
    config = described_class.parse(argv)

    expect(config).to eq('project' => 'test')
  end

  it 'parses a email option into a hash of configurations' do
    argv = ['-e', 'test']
    config = described_class.parse(argv)

    expect(config).to eq('email' => 'test')
  end

  it 'parses a email full option into a hash of configurations' do
    argv = ['--email', 'test']
    config = described_class.parse(argv)

    expect(config).to eq('email' => 'test')
  end

  it 'parses a config option into a hash of configurations' do
    argv = ['-c', 'spec/fixtures/file.yml']
    config = described_class.parse(argv)

    expectation = {
      'email'         => 'email',
      'password'      => 'password',
      'project'       => 'project',
      'start_at'      => '8',
      'lunch_at'      => '12',
      'restart_at'    => '13',
      'skip_defaults' => true,
      'leave_at'      => '17',
      'skips'         => []
    }
    expect(config).to eq(expectation)
  end

  it 'parses a config full option into a hash of configurations' do
    argv = ['--config', 'spec/fixtures/file.yml']
    config = described_class.parse(argv)

    expectation = {
      'email'         => 'email',
      'password'      => 'password',
      'project'       => 'project',
      'start_at'      => '8',
      'lunch_at'      => '12',
      'restart_at'    => '13',
      'skip_defaults' => true,
      'leave_at'      => '17',
      'skips'         => []
    }
    expect(config).to eq(expectation)
  end

  it 'raises ENOENT on invalid path to file' do
    argv = ['-c', 'wrong path to file']

    parser = described_class.new(argv)

    expect { parser.parse }.to raise_error(Errno::ENOENT)
  end

  it 'parses a skipped date option into a hash of configurations' do
    argv = ['-s', '13/06/1992,15/06/2002']
    config = described_class.parse(argv)

    expectation = {
      'skips' => [
        Date.new(1992, 6, 13), Date.new(2002, 6, 15)
      ]
    }
    expect(config).to eq(expectation)
  end

  it 'raises argument error in case of a wrong date format' do
    argv = ['-s', 'test']
    parser = described_class.new(argv)

    expect { parser.parse }.to raise_error(ArgumentError)
  end

  it 'parses a skipped date full option into a hash of configurations' do
    argv = ['--skip', '12/06/1992,12/06/1992']
    config = described_class.parse(argv)

    expectation = {
      'skips' => [
        Date.new(1992, 6, 12), Date.new(1992, 6, 12)
      ]
    }
    expect(config).to eq(expectation)
  end

  it 'parses options that override or fulfill config' do
    argv = ['-p', '3', '-e', '2', '-w', '1']
    config = described_class.parse(argv)

    expectation = {
      'password' => '3',
      'email'    => '2',
      'project'  => '1'
    }
    expect(config).to eq(expectation)
  end

  it 'raises on invalid arguments' do
    argv = ['-z']
    parser = described_class.new(argv)

    expect { parser.parse }.to raise_error(OptionParser::InvalidOption)
  end
end
